package de.cliffbreak.varo.managers;

import java.io.IOException;
import java.util.ArrayList;

import com.mojang.authlib.GameProfile;
import com.mojang.authlib.properties.Property;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.craftbukkit.v1_15_R1.CraftServer;
import org.bukkit.craftbukkit.v1_15_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_15_R1.entity.CraftPlayer;
import org.bukkit.entity.Player;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.scoreboard.Team;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import org.json.simple.JSONValue;
import org.json.simple.parser.ParseException;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.uitls.WebUtils;
import net.minecraft.server.v1_15_R1.DataWatcherObject;
import net.minecraft.server.v1_15_R1.DataWatcherRegistry;
import net.minecraft.server.v1_15_R1.EntityPlayer;
import net.minecraft.server.v1_15_R1.MinecraftServer;
import net.minecraft.server.v1_15_R1.PacketPlayOutEntityDestroy;
import net.minecraft.server.v1_15_R1.PacketPlayOutEntityHeadRotation;
import net.minecraft.server.v1_15_R1.PacketPlayOutEntityMetadata;
import net.minecraft.server.v1_15_R1.PacketPlayOutNamedEntitySpawn;
import net.minecraft.server.v1_15_R1.PacketPlayOutPlayerInfo;
import net.minecraft.server.v1_15_R1.PlayerConnection;
import net.minecraft.server.v1_15_R1.PlayerInteractManager;
import net.minecraft.server.v1_15_R1.WorldServer;
import net.minecraft.server.v1_15_R1.PacketPlayOutPlayerInfo.EnumPlayerInfoAction;

public class NPCManager {

    private final Varo plugin;
    private final ArrayList<EntityPlayer> players = new ArrayList<EntityPlayer>();
    private Scoreboard scoreboard;
    private Team afkTeam;

    public NPCManager(final Varo plugin) {
        this.plugin = plugin;
        this.scoreboard = Bukkit.getScoreboardManager().getMainScoreboard();
        if (scoreboard.getTeam("afk") == null) {

            this.afkTeam = scoreboard.registerNewTeam("afk");
        } else {
            this.afkTeam = scoreboard.getTeam("afk");
        }
        this.afkTeam.setPrefix("§8§l[AFK] ");
        this.afkTeam.setColor(ChatColor.GRAY);
    }

    public void createClone(final Player player, final boolean shouldHaveAFKPrefix) throws IOException {
        final Location location = player.getLocation();
        final MinecraftServer nmsServer = ((CraftServer) Bukkit.getServer()).getServer();
        final WorldServer nmsWorld = ((CraftWorld) player.getWorld()).getHandle();
        final GameProfile gameProfile = new GameProfile(player.getUniqueId(), player.getName());
        try {
            final String result = WebUtils.GET("https://sessionserver.mojang.com/session/minecraft/profile/"
                    + player.getUniqueId().toString().replace("-", "") + "?unsigned=false");
            final JSONObject resultObject = (JSONObject) JSONValue.parseWithException(result);
            final JSONObject properties = (JSONObject) ((JSONArray) resultObject.get("properties")).get(0);

            gameProfile.getProperties().put("textures", new Property("textures", properties.get("value").toString(),
                    properties.get("signature").toString()));
        } catch (IOException | ParseException e) {
            plugin.getLogger().info(e.getMessage());
            throw new IOException();
        }
        final EntityPlayer npc = new EntityPlayer(nmsServer, nmsWorld, gameProfile,
                new PlayerInteractManager(nmsWorld));

        npc.setLocation(location.getX(), location.getY(), location.getZ(), location.getYaw(), location.getPitch());

        if (shouldHaveAFKPrefix) {
            this.afkTeam.addEntry(npc.getName());
        }

        players.add(npc);

        for (Player connectionPlayer : Bukkit.getOnlinePlayers()) {
            final PlayerConnection connection = ((CraftPlayer) connectionPlayer).getHandle().playerConnection;
            connection.sendPacket(new PacketPlayOutPlayerInfo(EnumPlayerInfoAction.ADD_PLAYER, npc));
            connection.sendPacket(new PacketPlayOutNamedEntitySpawn(npc));
            connection.sendPacket(
                    new PacketPlayOutEntityHeadRotation(npc, (byte) ((location.getYaw() * 256.0F) / 360.0F)));
            npc.getDataWatcher().set(new DataWatcherObject<>(16, DataWatcherRegistry.a),
                    (byte) plugin.playerCache.getSkinParts(player.getUniqueId().toString().replace("-", "")));
            npc.getDataWatcher().set(new DataWatcherObject<>(17, DataWatcherRegistry.a),
                    plugin.playerCache.getMainHandAsByte(player.getUniqueId().toString().replace("-", "")));
            connection.sendPacket(new PacketPlayOutEntityMetadata(npc.getId(), npc.getDataWatcher(), true));
        }
    }

    public void syncClones(Player player) {
        for (EntityPlayer npc : this.players) {
            final PlayerConnection connection = ((CraftPlayer) player).getHandle().playerConnection;
            connection.sendPacket(new PacketPlayOutPlayerInfo(EnumPlayerInfoAction.ADD_PLAYER, npc));
            connection.sendPacket(new PacketPlayOutNamedEntitySpawn(npc));
            connection.sendPacket(new PacketPlayOutEntityHeadRotation(npc, (byte) ((npc.yaw * 256.0F) / 360.0F)));
            npc.getDataWatcher().set(new DataWatcherObject<>(16, DataWatcherRegistry.a),
                    (byte) plugin.playerCache.getSkinParts(player.getUniqueId().toString().replace("-", "")));
            npc.getDataWatcher().set(new DataWatcherObject<>(17, DataWatcherRegistry.a),
                    plugin.playerCache.getMainHandAsByte(player.getUniqueId().toString().replace("-", "")));
            connection.sendPacket(new PacketPlayOutEntityMetadata(npc.getId(), npc.getDataWatcher(), true));
        }
    }

    public void removeClone(Player player) {
        this.afkTeam.removeEntry(player.getName());
        for (int i = 0; i < this.players.size(); i++) {
            EntityPlayer npc = this.players.get(i);
            if (npc.getUniqueID().equals(player.getUniqueId())) {
                final WorldServer nmsWorld = ((CraftWorld) player.getWorld()).getHandle();
                nmsWorld.removeEntity(npc);

                for (Player connectionPlayer : Bukkit.getOnlinePlayers()) {
                    final PlayerConnection connection = ((CraftPlayer) connectionPlayer).getHandle().playerConnection;
                    connection.sendPacket(new PacketPlayOutPlayerInfo(EnumPlayerInfoAction.REMOVE_PLAYER, npc));
                    connection.sendPacket(new PacketPlayOutEntityDestroy(npc.getId()));
                }
                this.players.remove(npc);
            }
        }
    }
}